!>
! @license Apache-2.0
!
! Copyright (c) 2018 The Stdlib Authors.
!
! Licensed under the Apache License, Version 2.0 (the "License");
! you may not use this file except in compliance with the License.
! You may obtain a copy of the License at
!
!    http://www.apache.org/licenses/LICENSE-2.0
!
! Unless required by applicable law or agreed to in writing, software
! distributed under the License is distributed on an "AS IS" BASIS,
! WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
! See the License for the specific language governing permissions and
! limitations under the License.
!<

!> Wraps `dasum` as a subroutine.
!
! @param {integer} N - number of values to sum
! @param {Array<double>} dx - input array
! @param {integer} stride - stride length
! @param {double} sum - output variable reference
!<
subroutine dasumsub( N, dx, stride, sum )
  implicit none
  ! ..
  ! External functions:
  interface
    double precision function dasum( N, dx, stride )
      double precision :: dx(*)
      integer :: stride, N
    end function dasum
  end interface
  ! ..
  ! Scalar arguments:
  integer :: stride, N
  double precision :: sum
  ! ..
  ! Array arguments:
  double precision :: dx(*)
  ! ..
  ! Compute the sum of absolute values:
  sum = dasum( N, dx, stride )
  return
end subroutine dasumsub